﻿using System.Drawing;
using System;
using Wisej.Web;

namespace WiseJ_DataGridView
{
    public partial class Window1 : Page
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void Window1_Load(object sender, EventArgs e)
        {
            RefreshGrid();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            RefreshGrid();
        }

        private void RefreshGrid()
        {
            SetupGrid(grdData, 2, true, 24, DataGridViewEditMode.EditOnKeystrokeOrF2);

            DataGridViewColumn gCol;

            gCol = AddColumn2Grid(grdData, "FieldName", "Field Name", DataGridViewTriState.True, false, typeof(DataGridViewTextBoxColumn), DataGridViewContentAlignment.MiddleLeft, "", true);
            gCol.ReadOnly = true;
            gCol = AddColumn2Grid(grdData, "FieldType", "Field Type", DataGridViewTriState.True, false, typeof(DataGridViewTextBoxColumn), DataGridViewContentAlignment.MiddleLeft, "", true);
            gCol.ReadOnly = true;
            gCol = AddColumn2Grid(grdData, "Edit", "Editable Content", DataGridViewTriState.True, false, typeof(DataGridViewTextBoxColumn), DataGridViewContentAlignment.MiddleLeft, "", true);
            gCol.ReadOnly = false;

            DataGridViewRow dRow;
            DataGridViewCellStyle cStyle;
            Int32 iDP;

            for (int iLoop = 0; iLoop < 5; iLoop++)
            {
                dRow = AddGridRow("Textbox", "string");

                dRow.Cells["Edit"].ValueType = typeof(DataGridViewTextBoxCell);

                dRow = AddGridRow("Checkbox", "bool");

                DataGridViewCheckBoxCell cbCell = new DataGridViewCheckBoxCell();
                cbCell.ReadOnly = false;

                cStyle = new DataGridViewCellStyle();
                cStyle.Padding = new Padding(0);
                cStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
                dRow["Edit"] = cbCell;
                dRow["Edit"].Value = true;
                dRow["Edit"].Style = cStyle;

                iDP = Convert.ToInt32("13210".Substring(iLoop, 1));

                dRow = AddGridRow("Numeric " + iDP.ToString() + "dp", "numeric");

                DataGridViewNumericUpDownCell numCell = new DataGridViewNumericUpDownCell();
                numCell.DecimalPlaces = iDP;
                numCell.ReadOnly = false;
                numCell.Value = 12.12356;

                cStyle = new DataGridViewCellStyle();
                cStyle.Padding = new Padding(0);
                cStyle.Alignment = DataGridViewContentAlignment.TopRight;
                dRow["Edit"] = numCell;
                dRow["Edit"].Style = cStyle;
            }
        }

        private DataGridViewRow AddGridRow(string Fieldname, string Fieldtype)
        {
            grdData.Rows.Add(1);
            DataGridViewRow dRow = (DataGridViewRow)grdData.Rows[grdData.Rows.Count - 1];

            dRow["FieldName"].Value = Fieldname;
            dRow["FieldType"].Value = Fieldtype;

            return dRow;
        }

        private void grdData_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            string ErrorMessage = e.Exception.Message;

            if (ErrorMessage != String.Empty)
            {
                e.Cancel = false;
            }
            else { e.Cancel = false; }
        }
        private void grdData_CellBeginEdit(object sender, DataGridViewCellCancelEventArgs e)
        {
            int Col = e.ColumnIndex;
            int Row = e.RowIndex;

            if (!grdData.Columns[Col].Name.Equals("Edit", StringComparison.OrdinalIgnoreCase)) { e.Cancel = true; return; }

            DataGridViewCell Cell = grdData[e.ColumnIndex, e.RowIndex];
            DataGridViewColumn Column = grdData.Columns[Col];

            string CellType = grdData["FieldType", Row].Value.ToString();

            switch (CellType.ToLower())
            {
                case "string":
                    var textBox = new TextBox();
                    textBox.AutoCompleteList = new string[] { "One", "Two", "Three" };
                    textBox.Text = Cell.FormattedValue.ToString();
                    Column.Editor = textBox;
                    break;
                case "bool":
                    var checkBox = new CheckBox();
                    checkBox.Checked = IsValueTrue(Cell.Value.ToString());
                    checkBox.CheckAlign = System.Drawing.ContentAlignment.MiddleLeft;
                    Column.Editor = checkBox;
                    break;
                case "combobox":
                    String[] Options = grdData["SettingOptions", Row].Value.ToString().Split('|');
                    ComboBox cboBox = new ComboBox();
                    cboBox.Items.Clear();
                    cboBox.Items.AddRange(Options);
                    cboBox.Text = (string)Cell.FormattedValue;
                    Column.Editor = cboBox;
                    break;
                case "numeric":
                    NumericUpDown numUD = new NumericUpDown();
                    int DecimalPlaces = Convert.ToInt32(grdData["Fieldname", Row].Value.ToString().Substring(8,1));
                    numUD.TextAlign = HorizontalAlignment.Left;
                    numUD.HideUpDownButtons = true;
                    numUD.DecimalPlaces = DecimalPlaces;
                    numUD.Minimum = Int32.MinValue;
                    numUD.Maximum = Int32.MaxValue;
                    numUD.Value = Convert.ToDecimal(Convert2Double(Cell.FormattedValue));
                    Column.Editor = numUD;
                    break;
            }
        }

        private void grdData_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            int Col = e.ColumnIndex;
            int Row = e.RowIndex;
            DataGridViewCell Cell = grdData[e.ColumnIndex, e.RowIndex];
            DataGridViewColumn Column = grdData.Columns[Col];

            string CellType = grdData["FieldType", Row].Value.ToString();

            switch (CellType.ToLower())
            {
                case "string":
                    //TextBox tBox = new TextBox();
                    //tBox = (TextBox)Column.Editor;
                    //Cell.Value = tBox.Text;
                    break;
                case "bool":
                    CheckBox chkBox;
                    chkBox = (CheckBox)Column.Editor;
                    Cell.Value = (chkBox.Checked ? true : false);
                    break;
                case "combobox":
                    ComboBox cboBox = new ComboBox();
                    cboBox = (ComboBox)Column.Editor;
                    Cell.Value = cboBox.Text;
                    break;
                case "numeric":
                    NumericUpDown numUD = new NumericUpDown();
                    numUD = (NumericUpDown)Column.Editor;
                    Cell.Value = numUD.Value.ToString();
                    break;
            }
        }
        public void SetupGrid(DataGridView grd2Use, int iHeaderLines, bool bAutoSizeRows, int iDefaultRowHeight = 24, DataGridViewEditMode dgvEditMode = DataGridViewEditMode.EditProgrammatically)
        {
            grd2Use.EditMode = dgvEditMode;
            grd2Use.VirtualMode = false;
            grd2Use.RowCount = 0;
            grd2Use.ColumnCount = 0;
            grd2Use.MultiSelect = false;
            grd2Use.ShowColumnVisibilityMenu = false;
            grd2Use.ShowCellToolTips = true;

            grd2Use.DefaultCellStyle.Font = new Font("Tahoma", 9, FontStyle.Regular);

            grd2Use.ColumnHeadersHeight = (30 * iHeaderLines);
            grd2Use.ColumnHeadersDefaultCellStyle.WrapMode = DataGridViewTriState.True;

            grd2Use.DefaultCellStyle.Padding = new Padding(2, 4, 2, 2);
            if (iDefaultRowHeight > 0) { grd2Use.DefaultRowHeight = iDefaultRowHeight; }

            grd2Use.AutoSizeRowsMode = (bAutoSizeRows ? DataGridViewAutoSizeRowsMode.AllCellsExceptHeaders : DataGridViewAutoSizeRowsMode.None);
        }

        public DataGridViewColumn AddColumn2Grid(DataGridView grd2Use, string sColName, string sTitle, DataGridViewTriState triResize, bool bUseHTML, Type colDataType, DataGridViewContentAlignment cAlign, string sFormat, bool bWrapMode = false)
        {
            Wisej.Web.DataGridViewColumn col2Add = new Wisej.Web.DataGridViewColumn();

            col2Add.AllowHtml = bUseHTML;
            col2Add.Resizable = triResize;
            col2Add.ValueType = colDataType;
            col2Add.Name = sColName;
            col2Add.HeaderText = sTitle;
            col2Add.DefaultCellStyle.WrapMode = (bWrapMode ? DataGridViewTriState.True : DataGridViewTriState.NotSet);
            col2Add.DefaultCellStyle.Alignment = cAlign;
            col2Add.SortMode = DataGridViewColumnSortMode.Automatic;

            if (sFormat != "")
            {
                col2Add.DefaultCellStyle.Format = sFormat;
            }

            grd2Use.Columns.Add(col2Add);

            return col2Add;
        }
        public bool IsValueTrue(string sBoolean)
        { return (!(sBoolean == null) && "|Y|YES|TRUE|ON|1|".Contains(sBoolean.ToUpper())); }

        public double Convert2Double(object Number2Convert)
        {
            if (DBNull.Value.Equals(Number2Convert)) { return 0; }

            string sNewNo = Strip2Numeric(Number2Convert.ToString());

            if (sNewNo == String.Empty) { return 0; }

            return Convert.ToDouble(sNewNo);
        }
        private string Strip2Numeric(string sNum)
        {
            string sNew = "";

            for (int iChar = 0; iChar < sNum.Length; iChar++)
            {
                string sChar = sNum.Substring(iChar, 1);
                if ("-0123456789.".Contains(sChar)) { sNew += sChar; }
            }

            return sNew;
        }

    }
}
